/**
 * Copyright (c) 2015-present, JD.COM.
 * All rights reserved.
 */
'use strict';

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import StyleSheet from '../StyleSheet/StyleSheet';
import View from '../View/View';

class Modal extends Component {
  render(): ?ReactElement {
    if (this.props.visible === false) {
      if (this.shown) {
        this.props.onDismiss && this.props.onDismiss();
      }
      this.shown = false;
      return null;
    }else {
      if(!this.shown){
        this.props.onShow && this.props.onShow();
      }
    }

    this.shown = true;

    if (this.props.transparent) {
      var modalBackgroundColor = {backgroundColor: 'transparent'};
    }

    return (
      <View style={[styles.modal, modalBackgroundColor]} className="modal">
        <View style={[styles.container]}>
          {this.props.children}
        </View>
      </View>
    );
  }
}

Modal.propTypes = {
  animated: PropTypes.bool,
  transparent: PropTypes.bool,
  onDismiss: PropTypes.func,
};

let styles = StyleSheet.create({
  modal: {
    position: 'fixed',
    left: 0,
    right: 0,
    bottom: 0,
    top: 0,
    backgroundColor: '#fff',
    zIndex: 9999
  },
  container: {
    position: 'absolute',
    left: 0,
    right: 0,
    bottom: 0,
    top: 0,
  }
});

Modal.isReactNativeComponent = true;

export default Modal;
