// based on https://github.com/facebook/react/pull/4303/files

import normalizeNativeEvent from './normalizeNativeEvent';
import ReactDOM from 'react-dom';
import ReactDOMUnstableNativeDependencies from 'react-dom/unstable-native-dependencies';

const { EventPluginHub } = ReactDOM.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;
const { ResponderEventPlugin, ResponderTouchHistoryStore } = ReactDOMUnstableNativeDependencies;

function unsafeCastStringToDOMTopLevelType(topLevelType) {
    return topLevelType;
}
var TOP_MOUSE_DOWN = unsafeCastStringToDOMTopLevelType('mousedown');
var TOP_MOUSE_MOVE = unsafeCastStringToDOMTopLevelType('mousemove');
var TOP_MOUSE_UP = unsafeCastStringToDOMTopLevelType('mouseup');
var TOP_SCROLL = unsafeCastStringToDOMTopLevelType('scroll');
var TOP_SELECTION_CHANGE = unsafeCastStringToDOMTopLevelType('selectionchange');
var TOP_TOUCH_CANCEL = unsafeCastStringToDOMTopLevelType('touchcancel');
var TOP_TOUCH_END = unsafeCastStringToDOMTopLevelType('touchend');
var TOP_TOUCH_MOVE = unsafeCastStringToDOMTopLevelType('touchmove');
var TOP_TOUCH_START = unsafeCastStringToDOMTopLevelType('touchstart');

const endDependencies = [TOP_TOUCH_CANCEL, TOP_TOUCH_END, TOP_MOUSE_UP];
// const moveDependencies = [topTouchMove, topMouseMove];
const moveDependencies = [TOP_TOUCH_MOVE];
const startDependencies = [TOP_TOUCH_START, TOP_MOUSE_MOVE];


/**
 * Setup ResponderEventPlugin dependencies
 */
let eventTypes = ResponderEventPlugin.eventTypes;
['responderStart', 'responderMove', 'responderEnd', 'responderRelease',
'responderTerminationRequest', 'responderGrant', 'responderReject', 'responderTerminate'].forEach((type) => {
  let dependencies;
  if ('ontouchstart' in window) {
    dependencies = [
        TOP_TOUCH_START,
        TOP_TOUCH_CANCEL,
        TOP_TOUCH_END
    ];
  } else {
    // TODO: support move event
    dependencies = [
        TOP_MOUSE_DOWN,
        TOP_MOUSE_UP,
        TOP_MOUSE_MOVE
    ];
  }

  eventTypes[type].dependencies = dependencies;
});



eventTypes.moveShouldSetResponder.dependencies = moveDependencies;
eventTypes.selectionChangeShouldSetResponder.dependencies = [
    TOP_SELECTION_CHANGE
];
eventTypes.scrollShouldSetResponder.dependencies = [TOP_SCROLL];
eventTypes.startShouldSetResponder.dependencies = [TOP_TOUCH_START];



const originalRecordTouchTrack = ResponderTouchHistoryStore.recordTouchTrack;

ResponderTouchHistoryStore.recordTouchTrack = (topLevelType, nativeEvent) => {
  // Filter out mouse-move events when the mouse button is not down
  if (topLevelType === TOP_MOUSE_MOVE && !ResponderTouchHistoryStore.touchHistory.touchBank.length) {
    return;
  }

  const normalizedEvent = normalizeNativeEvent(nativeEvent);
  originalRecordTouchTrack.call(ResponderTouchHistoryStore, topLevelType, normalizedEvent);
};

EventPluginHub.injection.injectEventPluginsByName({
  ResponderEventPlugin
});