'use strict';

/**
 * Copyright (c) 2015-present, JD.COM.
 * All rights reserved.
 *
 * Copyright (c) 2015, Facebook, Inc.  All rights reserved.
 */
import PropTypes from 'prop-types';

import React from 'react';
import createReactClass from 'create-react-class';
import { Mixin as LayoutMixin } from '../Utilties/LayoutMixin';

const WebView = createReactClass({
  displayName: 'WebView',
  mixins: [LayoutMixin,],

  propTypes: {
    onLoadStart: PropTypes.func,
    onLoad: PropTypes.func,
    onLoadEnd: PropTypes.func,
    onError: PropTypes.func,
  },

  render() {
    return (
      <div style={{
        width: '100%',
        height: '100%',
        ...(this.props.style || {})
      }}>
        { this._renderContent() }
      </div>
    );
  },

  _renderContent() {
    let source = this.props.source;
    if (source.html) {
      return (
        <div dangerouslySetInnerHTML={{
          __html: source.html
        }} />
      )
    }
    let propsStyle = {};
    if(this.props.style) {
        if(this.props.style.width !== undefined) {
            propsStyle.width = this.props.style.width;
        }
        if(this.props.style.height !== undefined) {
            propsStyle.height = this.props.style.height;
        }
    }
    return (
      <iframe
        src={source.uri}
        onLoad={this.onLoad}
        onError={this.onError}
        frameBorder="0"
        scrolling={this.props.scrolling || 'yes'}
        style={{
          border: 'none',
          width: '100%',
          height: '100%',
          ...propsStyle
        }} />
    );
  },

  onLoad(e) {
    typeof this.props.onLoad === 'function' && this.props.onLoad(e);
    typeof this.props.onLoadEnd === 'function' && this.props.onLoadEnd(e);
  },

  onError(err) {
    typeof this.props.onLoad === 'function' && this.props.onLoad();
    typeof this.props.onError === 'function' && this.props.onError(err);
  },
});

WebView.isReactNativeComponent = true;

export default WebView;
